;****************************************************************
;* Titel :Basic procedures for DS1307 Realtime Clock Chip		*
;*        with AVR at 8MHz										*
;*                                                              *	
;* Version: 1                                                   *
;* Autor: Robert Ibener (r.ibener@aon.at)                       *
;* Date: 14/12/2008                                             *
;* Test Setup: STK500 and DS1307 as external Hardware.See		*
;*             DS1307 Datasheet for Hardware-Setup.				*
;*             Connect SQW-Pin of DS1307 to any LED on the		*
;*             STK500 to see the 1Hz-Signal.					* 	
;* Description:                                                 *
;* ===========													*
;* Initialize and read the Time/Date from the DS1307 RTC-Chip   *
;* Sends this package via RS232 to a PC.						*
;*																*
;* Any AVR with at least 64Bytes of SRAM could be used.			*
;* This code will be easyly transportable to any other AVRs	*
;* because it uses no special AVR-Features like Timers,USARTs	*
;* USI,TWI or Interrupts										*
;*																*
;* The I2C and RS232 timing is done by Delay-Loops,				*
;* so they MUST be adjusted for a Clock other than 8MHz!		*
;*																*
;* As this code will show only the basic procedures how to		*
;* handle the DS1307, additional code is required for a real	*
;* application.What is missing here is:							*
;* 1.)Set the time via a PC-Keyboard or any other keys			*
;* 2.)Use the Data from DS1307 for an Alarm-Clock				*
;* 3.)Display the Time on LCD,7-Segment or any other Displays	*
;* 3.)and so on													*
;*																*
;*                                                              *
;* Remark: Ignore WARNINGS after compiling.Some Registers has	*
;*         different names for the same register.				*
;*         But NO ERRORS should show up after compiling !!		*
;****************************************************************
.nolist
.include "tn2313def.inc"
.list
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;                         Define Registers
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

.def Work 		= r16	;general Registers
.def Work1 		= r17
.def Work2 		= r18
.def bitcnt 	= r16	;used in RS232.inc
.def TO_I2C 	= r22	;Byte sent to I2C
.def FROM_I2C 	= r23	;Byte received from I2C
.def Txbyte 	= r22	;Byte sent to RS232
.def Rxbyte 	= r23	;Byte received from RS232

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;                         Define Constants
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;Change the I2C-Port to your needs
.equ SCL  		= 1		;SCL on I2C_PORT,Pin 1
.equ SDA  		= 0		;SDA on I2C_PORT,Pin 0
.equ I2C_PORT 	= PORTB	;Port for I2C Out/In
.equ I2C_DDR	= DDRB	;DDR for I2C
.equ I2C_PIN	= PINB	;SDA-Pin

;Change the RS232-Port to your needs
;Used in "RS232.inc":
.equ RS232_In 	= PINB	
.equ RxD		= 2		;Receive pin is PB2
.equ RS232_Port = PORTB
.equ RS232_DDR	= DDRB	;DDR for RS232 In/Out	
.equ TxD		= 3		;Transmit pin is PB3
.equ sb			= 1		;Number of stop bits (1 or 2)
.equ b			= 135	;Delay-Constant for 9600 bps @ 8 MHz

;----------------------
;Used in "Conversion.inc"
.equ Year20xx	= 0x20	;Year 20xx.This is done because DS1307
						;shows the year only in 00-99 format.
						;Sorry : Your children must change
						;this constant for year 2100 and
						;re-assemble the whole stuff!!

;*************************************************************
;* Next constants will be transferred to simulate Date/Time-Setting
;* 			---> Set Values to your needs <---
;* 		 Values below must be in packed BCD-Format !
;*************************************************************

.equ _Seconds	= 0x30		;Seconds=30
.equ _Minutes	= 0x35		;Minutes=35
.equ _Hours		= 0x15		;Hours=15 (24-Hour Mode)
.equ _Day		= 0x03		;Day-of-Week (1=Sunday,2=Monday...)
.equ _Date		= 0x16		;Day
.equ _Month		= 0x12		;December
.equ _Year		= 0x08		;2008

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;                         Define Macros
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

; No Macros

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;                        Interrupt-Vectors
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

.CSEG
.ORG 0x0000			

	rjmp RESET 	;Reset Handler
	reti		;External Int. 0 Handler
	reti		;External Int. 1 Handler
	reti		;Timer1 Capture Handler
	reti		;Timer1 Compare A Handler
	reti		;Timer1 Overflow Handler
	reti		;Timer0 Overflow Handler
	reti		;USART RX complete Handler
	reti		;USART UDR empty Handler
	reti		;USART TX complete Handler
	reti		;Analog Comparator Handler
	reti		;Pin change Interrupt
	reti		;Timer1 Compare B Handler
	reti		;Timer0 Compare A Handler
	reti		;Timer0 Compare B Handler
	reti		;USI Start Handler
	reti		;USI Overflow Handler
	reti		;EEPROM Ready Handler
	reti		;Watchdog Overflow Handler
reset:
;	ldi r16,0x56			;Load OSCCAL-Value
;	out OSCCAL,r16
	ldi r16,RAMEND			;init Stack
	out spl,r16
	sbi RS232_DDR,TxD		;Define TxD as RS232 Output
	rcall I2CBus_Init		;silence the I2C-Bus
	rcall DS1307_Init		;set Date/Time of DS1307
loop1:
	rcall Read_registers	;transfer Date/Time from DS1307 to SRAM
	rcall BCD2ASCII			;Converts packed BCD to ASCII
	rcall Show_Date			;Show Date/Time on RS232

;-----------------------------------------
; Delay 2 sec. for RS232-Output
; should be removed in a real application
;-----------------------------------------
	ldi r18,0x64
del_:
	rcall _wait20
	dec r18
	brne Del_
	rjmp loop1

_wait20:			;wait 20msec
	push r17
	push r18
       ldi  R17,$E1
_w20_0:
	ldi  R18,$EA
_w20_1:
	dec  R18
	brne _w20_1
	dec  R17
	brne _w20_0
	pop r18
	pop r17
	ret

.cseg
;-------------------------------------------
; Define Day-of-Week Constants
; Note : First Character is on LSB !
;        Characters are defined for
;        german language.Change as you like!
;-------------------------------------------
Sun: .dw 0x6F53	;So(nntag)
Mon: .dw 0x6F4D	;Mo(ntag)
Die: .dw 0x6944 ;Di(enstag)
Mit: .dw 0x694D ;Mi(ttwoch)
Don: .dw 0x6F44 ;Do(nnerstag)
Fre: .dw 0x7246 ;Fr(eitag)
Sam: .dw 0x6153 ;Sa(mstag)


;-------------------------------------------
;  Files to include :
;-------------------------------------------
.include "DS1307.inc"		;DS1307 specific routines
.include "RS232.inc"		;RS232 In/Out Routines
.include "I2C_Sub_8MHz.inc"	;Basic I2C-Routines
.include "Conversion.inc"	;Converts packed BCD to ASCII




