;*******************************************
;*******************************************
;
; General I2C Routines
;
;*******************************************
;*******************************************
;
; Register,Port,Pin Definitions:
; (can be changed in the main-routine)
;.equ SCL  		= 1		;SCL on PB1
;.equ SDA  		= 0		;SDA on PB0
;.equ I2C_PORT	= PORTB	;Port for I2C Out/In
;.equ I2C_DDR	= DDRB	;Direction Register
;.equ I2C_PIN	= PINB	;read on SDA-Pin
;
;.def TO_I2C 	= r22	;Byte sent to I2C
;.def FROM_I2C 	= r23	;Byte received from I2C

;-------------------------------------------
; I2CBusInit
;-------------------------------------------
; Initialize I2C interface ports
; Uses : -
; Destroys :-
;-------------------------------------------

I2CBus_Init:
	cbi  I2C_PORT,SCL 	; SCL-Pin only drives low if driven
	cbi	 I2C_PORT,SDA	; SDA-Pin only drives low if driven
	cbi	 I2C_DDR,SDA	; SDA high-z
	cbi	 I2C_DDR,SCL	; SCL high-z
;-------------------------------------------
; I2CStop
;-------------------------------------------
; Send I2C stop condition
; Uses : -
; Destroys :-
;-------------------------------------------

I2C_Stop:
	sbi	I2C_DDR,SDA	; Drive SDA low
	rcall _wait005
	cbi	I2C_DDR,SCL	; SCL high z
	rcall _wait005
	cbi	I2C_DDR,SDA	; SDA high z
	ret
	
;-------------------------------------------
; I2CStart
;-------------------------------------------
; Send I2C Start + Data from the TO_I2C-Register
; to the I2C-Bus.
; Uses: Work1,TO_I2C,T-Bit in SREG
; Destroys: Work1,TO_I2C,T-Bit in SREG
;-------------------------------------------

I2C_Start:
	cbi	 I2C_DDR,SDA	; SDA high-z
	rcall _wait005
	cbi	 I2C_DDR,SCL	; Clock high-z
	rcall _wait005
	sbi	 I2C_DDR,SDA	; Drive SDA low
	rcall _wait005
	sbi	 I2C_DDR,SCL	; Drive SCL low

;-------------------------------------------
; I2CputByte
;-------------------------------------------
; Sends 8 bit data in TO_I2C to the I2C-
; Device, checks for an ACK at the end.
; Returns SREG Bit "T" set if no ACK received.
; The "T"-Bit can be checked afterwards for
; a "Device not responding"-Routine somewhere
; else in your code.
;
; Uses: Work1,TO_I2C,T-Bit in SREG
; Destroys: Work1,TO_I2C,T-Bit in SREG
;-------------------------------------------

I2C_PutByte:
	ldi	  Work1,8			;load Bit-Counter
I2CputBits:
	add	  TO_I2C,TO_I2C
	brcc  I2CputBits0
	cbi   I2C_DDR,SDA		;send 1 ( SDA high z)
	rcall _wait005
	rjmp  I2CputBits1
I2CputBits0:
	sbi	  I2C_DDR,SDA		;send 0 (SDA drives low)
	rcall _wait005
I2CputBits1:	
	cbi	  I2C_DDR,SCL		;Clock high z
	rcall _wait005
	sbi	  I2C_DDR,SCL		;Drive SCL low
rcall _wait005
	dec	  Work1
	brne  I2CputBits
	cbi   I2C_DDR,SDA		;SDA high z
	rcall _wait005
	cbi	  I2C_DDR,SCL		;SCL high z
	rcall _wait005
getack_:					;Check for ACK
	clt						;Clear Error-Flag (Bit "T" in SREG)
	sbic  PINB, SDA			;if SDA=high (no ACK) then
	set						;set T-Bit in SREG as Error-Flag
	sbi	  I2C_DDR,SCL		;Drive SCL low
	ret

;-------------------------------------------
; I2CgetByte
;-------------------------------------------
; Note:Dont be confused with the "T"-Bit function
;      in the Subroutine "I2C_PutByte" with the
;      "T"-Bit function here.The "T"-Bit in this
;      routine has a complete other meaning !
;
; If "T"-Bit in SREG is clear:
; Read a byte from the adressed I2C-Device into "From_I2C".
; After receiving the Byte,transmitt "ACK" to the I2C-Device.
;
; If "T"-Bit in SREG is set:
; Read a byte from the adressed I2C-Device into "From_I2C".
; After receiving the Byte,transmitt "NACK" and "STOP"
; to the I2C-Device
; 
; Uses: Work1,From_I2C,T-Bit in SREG
; Destroys: Work1,From_I2C,T-Bit in SREG
;-------------------------------------------

I2C_GetByte:
	ldi	 	Work1,8			;load Bit-Counter
	clr	 	From_I2C
	cbi	 	I2C_DDR,SDA		;SDA high z
	rcall 	_wait005
I2CgetBits:
	cbi	 	I2C_DDR,SCL		;SCL high z
	rcall 	_wait005
	add	 		From_I2C,From_I2C
	sbic 	I2C_PIN,SDA		;if SDA=0 skip increment
	inc  	From_I2C		;update received Bit
	sbi	 	I2C_DDR,SCL		;drive SCL low
	rcall 	_wait005
	dec	 	Work1
	brne 	I2CgetBits
	brts	I2C_Nack		;last byte to send
;-------------------------------------------
; Send "ACK" to I2C-Device
;-------------------------------------------
; Uses : -
; Destroys :-
;-------------------------------------------
I2C_ACK:
	sbi  	I2C_DDR,SDA		;send ACK (SDA drives low)
	rcall 	_wait005
	cbi	 	I2C_DDR,SCL		;SCL high z
	rcall 	_wait005
	sbi	 	I2C_DDR,SCL		;Drive SCL low
	rcall 	_wait005
	cbi	 	I2C_DDR,SDA		;release bus (SDA high z)
	rcall 	_wait005
	ret

;-------------------------------------------
; Send "NACK" and "STOP" to the I2C-Device
;-------------------------------------------
; Uses : -
; Destroys :-
;-------------------------------------------
I2C_Nack:				
	cbi		I2C_DDR,SDA			;SDA high z
	rcall 	_wait005
	cbi		I2C_DDR,SCL			;SCL high z
	rcall 	_wait005 
	sbi		I2C_DDR,SCL			;Drive SCL low
	rcall 	_wait005
	rjmp 	I2C_Stop			;"ret" is in I2C_Stop

;*******************************************
;I2C Delay subroutine for 8Mhz
;-------------------------------------------
;  Used Registers: r17,r18
;  Registers destroyed : none
;*******************************************

_wait005:			;wait 5usec
		push r17
		ldi  R17,$09
_w005:
		dec  R17
		brne _w005
		pop r17
		ret
